/***

instances.c - handler routines for multiple IPTraf instances

Copyright (c) Gerard Paul Java 1998

This software is open source; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License in the included COPYING file for
details.

***/

#include "portable.h"
#include "error.h"
#include "dirs.h"

extern char active_facility_lockfile[64];
extern int daemonized;

void mark_facility(char *tagfile, char *facility)
{
#ifndef MSDOS
    int fd;
    char errstring[80];

    fd = open(tagfile, O_CREAT | O_WRONLY | O_TRUNC, S_IRUSR | S_IWUSR);
    if (fd < 0) {
	sprintf(errstring, "Warning: unable to tag %s", facility);
	write_error(errstring, daemonized);
    }
    close(fd);
    strcpy(active_facility_lockfile, tagfile);
#endif
}

void unmark_facility(char *tagfile)
{
#ifndef MSDOS
    unlink(tagfile);
    strcpy(active_facility_lockfile, "");
#endif
}

int facility_active(char *tagfile)
{
#ifdef MSDOS
    ARGSUSED (tagfile);
    return (0);
#else
    int fd;

    fd = open(tagfile, O_RDONLY);

    if (fd < 0)
	return 0;
    else {
	close(fd);
	return 1;
    }
#endif
}

/*
 * Increments or decrements the process count
 */
 
void adjust_process_count(int inc)
{
#ifndef MSDOS
    int fd;
    int proccount = 0;
    int brw;
    
    fd = open(PROCCOUNTFILE, O_RDWR | O_CREAT, S_IRUSR | S_IWUSR);
    brw = read(fd, &proccount, sizeof(int));
    if (brw == 0)
        proccount = 0;
        
    proccount += inc;
    
    lseek(fd, 0, SEEK_SET);
    brw = write(fd, &proccount, sizeof(int));
    close(fd);
#endif
}

/*
 * Returns TRUE if this is the last instance, and is therefore responsible
 * for restoring the promicuous states saved by the first instance.
 *
 * Man, this is getting more complex by the minute :)
 */
 
int is_last_instance(void)
{
#ifdef MSDOS
    return (1);
#else
    int fd;
    int proccount = 0;
    int br;
    
    fd = open(PROCCOUNTFILE, O_RDONLY);
    br = read(fd, &proccount, sizeof(int));
    close(fd);
    return ((proccount == 1) || (br < 0) || fd < 0);
#endif
}

/*
 * Returns TRUE if no facilities are currently running in other instances of
 * IPTraf.  Call this before the first invocation of adjust_process_count(1)
 */
 
int first_active_facility(void)
{
#ifdef MSDOS
    return (1);
#else
    int fd;
    int proccount = 0;
    int br;
    
    fd = open(PROCCOUNTFILE, O_RDONLY);
    br = read(fd, &proccount, sizeof(int));
    close(fd);
    return ((proccount == 0) || (br < 0) || (fd < 0));
#endif
}
    
