/*
 * Portability hacks for IPTraf on Unixes and djgpp (MSDOS)
 */

#ifndef __PORTABLE_H
#define __PORTABLE_H

#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <signal.h>
#include <ctype.h>
#include <errno.h>

#if defined(__DJGPP__) || defined(unix) || defined(__linux__)
  #include <unistd.h>
  #include <sys/time.h>       /* struct timeval */
  #include <sys/wait.h>
#endif

#if defined(MSDOS)
  #include <stdarg.h>
  #include <time.h>
  #include <memory.h>
  #include <tcp.h>

  #include "if_fddi.h"
  #include "if_ether.h"
  #include "icmp.h"

  extern int snprintf (char *buf, size_t len, const char *fmt, ...);
  extern int vsnprintf(char *buf, size_t len, const char *fmt, va_list args);
  
  #define SOCK_SELECT select_s
  #define SOCK_READ   read_s
  #define SOCK_WRITE  write_s
  #define SOCK_CLOSE  close_s
  #define SOCK_IOCTL  ioctlsocket

#else
  #include <unistd.h>

  #define SOCK_SELECT select
  #define SOCK_READ   read
  #define SOCK_WRITE  write
  #define SOCK_CLOSE  close
  #define SOCK_IOCTL  ioctl
#endif

#if defined(unix) && defined(SYSV)
  #include <sys/times.h>
  #include <sys/param.h>
#endif

#include <fcntl.h>

#include <sys/ioctl.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <net/if.h>
#include <net/if_arp.h>

#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/tcp.h>
#include <netinet/udp.h>
#include <netinet/ip_icmp.h>

#include <arpa/inet.h>
#include <netdb.h>

#if defined(USE_LIBPCAP)
#include <pcap.h>
#endif

/* These cannot be included multiple times.
 * Uses PDcurses on MSDOS
 */
#include <curses.h> 
#include <panel.h>

#if defined(__linux__)
  #include <paths.h>     /* needed? */
  #include <endian.h>
  #include <asm/types.h>
  #include <sys/wait.h>

  #include <linux/if_packet.h>
  #include <linux/if_ether.h>
  #include <linux/if_fddi.h>
  #include <linux/isdn.h>
  #include <linux/sockios.h>

#elif defined(__MSDOS__) || defined(__i386__) || defined(__386__)
  #undef  __BYTE_ORDER
  #undef  __LITTLE_ENDIAN
  #undef  __BIG_ENDIAN
  #define __BYTE_ORDER     1234
  #define __LITTLE_ENDIAN  1234
  #define __BIG_ENDIAN     4321
#endif

#define SCROLLUP   0
#define SCROLLDOWN 1

#ifndef MAXHOSTNAMELEN
#define MAXHOSTNAMELEN  64
#endif

#ifndef min
#define min(a, b) ((a > b) ? b : a)
#endif

#ifndef max
#define max(a, b) ((a > b) ? a : b)
#endif

#ifndef ARGSUSED
#define ARGSUSED(x) ((void)x)
#endif

#ifndef DIM
#define DIM(x) (sizeof(x) / sizeof(x[0])) /* array dimension */
#endif

#ifndef ETH_ALEN
#define ETH_ALEN   6
#endif
#ifndef ETH_HLEN
#define ETH_HLEN   (ETH_ALEN+ETH_ALEN+2)
#endif

#ifndef ETH_P_IP
#define ETH_P_IP   0x0800
#endif

#ifndef ETH_P_ARP
#define ETH_P_ARP  0x0806
#endif

#ifndef ETH_P_RARP
#define ETH_P_RARP 0x8035
#endif

#ifndef ICMP_DEST_UNREACH
#define ICMP_DEST_UNREACH 3
#endif

#if !defined(__linux__)
  struct iphdr {
#if __BYTE_ORDER == __LITTLE_ENDIAN
         u_char     ihl     : 4;
         u_char     version : 4;
#elif __BYTE_ORDER == __BIG_ENDIAN
         u_char     version : 4;
         u_char     ihl     : 4;
#endif
         u_char     tos;
         u_short    tot_len;
         u_short    id;
         u_short    frag_off;
         u_char     ttl;
         u_char     protocol;
         u_short    check;
         u_int32_t  saddr;
         u_int32_t  daddr;
         /* The options start here. */
       };

  #if 0
  struct icmphdr {
         u_char  type;
         u_char  code;
         u_short cksum;
         u_char  data[4];  /* at least 4 bytes */
       };
  #endif

#endif

/*
 * Macros for accessing tcphdr fields on Linux and BSD derived stacks
 * Only those we need.
 */
#if defined(__linux__)
  #define TCP_SRC_PORT(tcp)  (tcp)->source
  #define TCP_DST_PORT(tcp)  (tcp)->dest
  #define TCP_DATA_OFS(tcp)  ((tcp)->doff << 2)
  #define TCP_SYN_FLAG(tcp)  (tcp)->syn
  #define TCP_FIN_FLAG(tcp)  (tcp)->fin
  #define TCP_SEQ_NUM(tcp)   ntohl ((tcp)->seq)
#else
  #define TCP_SRC_PORT(tcp)  (tcp)->th_sport
  #define TCP_DST_PORT(tcp)  (tcp)->th_dport
  #define TCP_DATA_OFS(tcp)  ((tcp)->th_off << 2)
  #define TCP_SYN_FLAG(tcp)  ((tcp)->th_flags & TH_SYN)
  #define TCP_FIN_FLAG(tcp)  ((tcp)->th_flags & TH_FIN)
  #define TCP_SEQ_NUM(tcp)   ntohl ((tcp)->th_seq)
#endif

/*
 * Same hacks for udphdr
 * Only those we need.
 */
#if defined(__linux__)
  #define UDP_SRC_PORT(udp)  (udp)->source
  #define UDP_DST_PORT(udp)  (udp)->dest
#else
  #define UDP_SRC_PORT(udp)  (udp)->uh_sport
  #define UDP_DST_PORT(udp)  (udp)->uh_dport
#endif

/*
 * We hack SOCK_PACKET emualtion using the libpcap API.
 */
#if defined(USE_LIBPCAP)
  struct sockaddr_ll {
         u_short sll_family;
         u_short sll_hatype;
         u_short sll_protocol;
         u_short sll_pkttype;
         u_char  sll_data[12];
       };

  #define PACKET_OUTGOING 1
#endif

extern int exitloop;
extern int daemonized;

/*
 * DOS doesn't support multiple processes (fork).
 * Patch up a simple resolver (or none at all)
 */
#ifdef MSDOS
  #define NOTRESOLVED 0
  #define RESOLVING   1
  #define RESOLVED    2

  struct res_buffer {
         int    type;
         int    ready;
         struct in_addr saddr;
         char   fqdn[45];
       };
#else
  #include "rvnamed.h"
#endif

#endif /* __PORTABLE_H */
